@@ -3,9 +3,12 @@
 import datetime
 import importlib.metadata
 import json
+import os
 import sys
+import termios
 import threading
 import time
+import tty
 
 import click
 import requests
@@ -268,6 +271,413 @@ def handle_api_error(error):
     sys.exit(1)
 
 
+# Interactive pagination helpers (reused for list -i)
+def _get_terminal_size():
+    try:
+        return os.get_terminal_size()
+    except Exception:
+        return os.terminal_size((80, 24))
+
+
+def _get_keypress():
+    try:
+        fd = sys.stdin.fileno()
+        old_settings = termios.tcgetattr(fd)
+        try:
+            tty.setraw(sys.stdin.fileno())
+            while True:
+                ch = sys.stdin.read(1)
+                if ch == "\x1b":
+                    try:
+                        next_chars = sys.stdin.read(2)
+                        seq = ch + next_chars
+                        if seq == "\x1b[D":
+                            return "left"
+                        elif seq == "\x1b[C":
+                            return "right"
+                        elif seq == "\x1b[A":
+                            return "up"
+                        elif seq == "\x1b[B":
+                            return "down"
+                        continue
+                    except Exception:
+                        continue
+                elif ch.lower() in ["q", "h", "l", "p", "n", "j", "k", "s", "m"]:
+                    return ch.lower()
+                elif ch == "\x03":
+                    return "\x03"
+                else:
+                    continue
+        finally:
+            termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
+    except Exception:
+        # Fallback prompt
+        while True:
+            try:
+                key = input(
+                    "Enter command (←/→ page, ↑/↓ scroll, m metadata, q quit): "
+                ).lower()
+                if key in [
+                    "left",
+                    "right",
+                    "up",
+                    "down",
+                    "q",
+                    "h",
+                    "l",
+                    "p",
+                    "n",
+                    "j",
+                    "k",
+                    "s",
+                    "m",
+                ]:
+                    return key
+            except KeyboardInterrupt:
+                return "q"
+
+
+def _spinner_loop(message="Loading"):
+    spinner_chars = "|/-\\"
+    idx = 0
+    while getattr(_spinner_loop, "running", False):
+        print(
+            f"\r{message}... {spinner_chars[idx % len(spinner_chars)]}",
+            end="",
+            flush=True,
+        )
+        idx += 1
+        time.sleep(0.1)
+    print("\r" + " " * (len(message) + 10) + "\r", end="", flush=True)
+
+
+def _start_spinner(message="Loading"):
+    _spinner_loop.running = True
+    t = threading.Thread(target=_spinner_loop, args=(message,))
+    t.daemon = True
+    t.start()
+    return t
+
+
+def _stop_spinner():
+    _spinner_loop.running = False
+    time.sleep(0.2)
+
+
+def _interactive_pagination(
+    get_data_func,
+    format_item_func,
+    items_attr: str,
+    title: str,
+    limit: int = 50,
+    initial_page: int = 1,
+    enable_search: bool = False,
+    on_search=None,
+    search_summary_fn=None,
+    **kwargs,
+):
+    page = max(1, int(initial_page or 1))
+    scroll_offset = 0
+    # Enter alt screen
+    click.echo("\033[?1049h", nl=False)
+    click.echo("\033[?47h", nl=False)
+    current_items = []
+    total = 0
+    try:
+        while True:
+            term = _get_terminal_size()
+            height = term.lines
+            width = term.columns
+            content_height = max(5, height - 12)
+
+            if not current_items:
+                spinner = _start_spinner("Fetching data")
+                try:
+                    result = get_data_func(page=page, limit=limit, **kwargs)
+                    current_items = getattr(result, items_attr, [])
+                    total = getattr(result, "total", len(current_items))
+                    scroll_offset = 0
+                finally:
+                    _stop_spinner()
+
+            total_pages = (total + limit - 1) // limit if total > 0 else 1
+            has_prev = page > 1
+            has_next = page < total_pages
+
+            total_items_on_page = len(current_items)
+            max_scroll = max(0, total_items_on_page - content_height)
+            scroll_offset = max(0, min(scroll_offset, max_scroll))
+
+            click.echo("\033[2J\033[H", nl=False)
+            click.echo("=" * min(80, width))
+            click.echo(f"{title} (Interactive Mode)")
+            click.echo("=" * min(80, width))
+
+            # Optional search summary display
+            if enable_search and callable(search_summary_fn):
+                summary = search_summary_fn()
+                if summary:
+                    click.echo(f"🔍 Filter: {summary}")
+                    click.echo()
+
+            scroll_info = ""
+            if total_items_on_page > content_height:
+                scroll_info = f" | Scroll: {scroll_offset + 1}-{min(scroll_offset + content_height, total_items_on_page)} of {total_items_on_page}"
+            click.echo(
+                f"📄 Page {page} of {total_pages} (Total: {total} items{scroll_info})"
+            )
+            click.echo("-" * min(80, width))
+
+            if not current_items:
+                click.echo()
+                click.echo("   No results found.")
+                click.echo()
+            else:
+                click.echo()
+                visible = current_items[scroll_offset : scroll_offset + content_height]
+                for item in visible:
+                    line = format_item_func(item)
+                    if len(line) > width - 1:
+                        line = line[: width - 4] + "..."
+                    click.echo(line)
+                for _ in range(content_height - len(visible)):
+                    click.echo()
+
+            click.echo("-" * min(80, width))
+            nav = []
+            nav.append(
+                "← Previous page (left/h)" if has_prev else "← Previous page (disabled)"
+            )
+            nav.append(
+                "→ Next page (right/l)" if has_next else "→ Next page (disabled)"
+            )
+            if total_items_on_page > content_height:
+                nav.append(
+                    "↑ Scroll up (up/k)"
+                    if scroll_offset > 0
+                    else "↑ Scroll up (disabled)"
+                )
+                nav.append(
+                    "↓ Scroll down (down/j)"
+                    if scroll_offset < max_scroll
+                    else "↓ Scroll down (disabled)"
+                )
+            if enable_search:
+                nav.append("[m]etadata")
+            nav_text = " | ".join(nav)
+            if len(nav_text) > width - 1:
+                mid = len(nav) // 2
+                click.echo(f"Navigation: {' | '.join(nav[:mid])}")
+                click.echo(f"            {' | '.join(nav[mid:])}")
+            else:
+                click.echo(f"Navigation: {nav_text}")
+            click.echo()
+            if enable_search:
+                click.echo(
+                    "💡 Use arrow keys to navigate, 'm' to set metadata filter, 'q' to quit..."
+                )
+            else:
+                click.echo("💡 Use arrow keys to navigate, 'q' to quit...")
+
+            key = _get_keypress()
+            if key in ("left", "h", "p"):
+                if has_prev:
+                    page -= 1
+                    current_items = []
+            elif key in ("right", "l", "n"):
+                if has_next:
+                    page += 1
+                    current_items = []
+            elif key in ("up", "k"):
+                if scroll_offset > 0:
+                    scroll_offset -= 1
+            elif key in ("down", "j"):
+                if scroll_offset < max_scroll:
+                    scroll_offset += 1
+            elif key in ("m", "s") and enable_search and callable(on_search):
+                # Temporarily exit alt screen for input
+                click.echo("\033[?1049l", nl=False)
+                try:
+                    changed = on_search()
+                finally:
+                    # Re-enter alt screen
+                    click.echo("\033[?1049h", nl=False)
+                # Reset page and refresh on any change
+                page = 1
+                current_items = []
+            elif key in ("q", "\x03"):
+                break
+
+    finally:
+        # Exit alt screen
+        click.echo("\033[?1049l", nl=False)
+
+
+# ─────────────────────────────────────────────────────────────
+#  User Commands
+# ─────────────────────────────────────────────────────────────
+
+
+@cli.group(cls=AliasedGroup)
+def user():
+    """Manage settings for the current authenticated user."""
+    pass
+
+
+# ── API Keys ────────────────────────────────────────────────
+
+
+@user.group(cls=AliasedGroup, name="api-key")
+def user_api_key():
+    """Manage user API keys."""
+    pass
+
+
+@user_api_key.command(name="list", aliases=["ls"])
+@click.option("--json", "json_mode", is_flag=True, default=False)
+def user_api_key_list(json_mode):
+    client = get_client()
+    try:
+        keys = client.user.list_api_keys()
+        if json_mode:
+            for k in keys:
+                click.echo(format_json(k))
+        else:
+            headers = ["ID", "Prefix", "Created", "Last Used"]
+            rows = [
+                [
+                    k.id,
+                    k.key_prefix,
+                    unix_timestamp_to_datetime(k.created),
+                    unix_timestamp_to_datetime(k.last_used),
+                ]
+                for k in keys
+            ]
+            print_docker_style_table(headers, rows)
+    except Exception as e:
+        handle_api_error(e)
+
+
+@user_api_key.command(name="create", aliases=["new"])
+@click.option("--json", "json_mode", is_flag=True, default=False)
+def user_api_key_create(json_mode):
+    client = get_client()
+    try:
+        with Spinner(
+            text="Creating API key...",
+            success_text="API key created",
+            success_emoji="🔑",
+        ):
+            resp = client.user.create_api_key()
+        if json_mode:
+            click.echo(format_json(resp))
+        else:
+            click.secho("API key created.", fg="green")
+            click.echo(f"ID: {resp.id}")
+            click.echo(f"Prefix: {resp.key_prefix}")
+            click.echo(f"Created: {unix_timestamp_to_datetime(resp.created)}")
+            click.secho(
+                "This is your API key — it will not be shown again:", fg="yellow"
+            )
+            click.secho(resp.key, fg="yellow")
+    except Exception as e:
+        handle_api_error(e)
+
+
+@user_api_key.command(name="delete", aliases=["rm"])
+@click.argument("api_key_id")
+def user_api_key_delete(api_key_id):
+    client = get_client()
+    try:
+        with Spinner(
+            text=f"Deleting API key {api_key_id}...",
+            success_text="API key deleted",
+            success_emoji="🗑",
+        ):
+            client.user.delete_api_key(api_key_id)
+    except Exception as e:
+        handle_api_error(e)
+
+
+# ── SSH Key ─────────────────────────────────────────────────
+
+
+@user.group(cls=AliasedGroup, name="ssh-key")
+def user_ssh_key():
+    """Manage user SSH public key."""
+    pass
+
+
+@user_ssh_key.command(name="get")
+@click.option("--json", "json_mode", is_flag=True, default=False)
+def user_ssh_key_get(json_mode):
+    client = get_client()
+    try:
+        info = client.user.get_ssh_key()
+        if json_mode:
+            click.echo(format_json(info))
+        else:
+            headers = ["Public Key", "Created"]
+            rows = [[info.public_key, unix_timestamp_to_datetime(info.created)]]
+            print_docker_style_table(headers, rows)
+    except Exception as e:
+        handle_api_error(e)
+
+
+@user_ssh_key.command(name="set", aliases=["update"])
+@click.option(
+    "--public-key",
+    required=True,
+    help="SSH public key string (e.g., 'ssh-rsa AAAA...')",
+)
+@click.option("--json", "json_mode", is_flag=True, default=False)
+def user_ssh_key_set(public_key, json_mode):
+    client = get_client()
+    try:
+        with Spinner(
+            text="Updating SSH key...",
+            success_text="SSH key updated",
+            success_emoji="🔐",
+        ):
+            info = client.user.update_ssh_key(public_key)
+        if json_mode:
+            click.echo(format_json(info))
+        else:
+            click.secho("SSH key updated.", fg="green")
+            click.echo(f"Created: {unix_timestamp_to_datetime(info.created)}")
+    except Exception as e:
+        handle_api_error(e)
+
+
+# ── Usage ───────────────────────────────────────────────────
+
+
+@user.command(name="usage")
+@click.option("--interval", default=None, help="Lookback window (e.g., 30m, 3h, 7d)")
+@click.option("--json", "json_mode", is_flag=True, default=False)
+def user_usage(interval, json_mode):
+    client = get_client()
+    try:
+        usage = client.user.usage(interval=interval)
+        if json_mode:
+            click.echo(format_json(usage))
+        else:
+            click.secho("User usage overview", fg="green")
+            click.echo(f"Interval buckets: {len(usage.items)}")
+            if usage.instance:
+                last = usage.instance[-1]
+                click.echo(
+                    f"Last bucket instance CPU: {last.instance_cpu_time}, MEM: {last.instance_memory_time}, DISK: {last.instance_disk_time}"
+                )
+            if usage.snapshot:
+                last = usage.snapshot[-1]
+                click.echo(
+                    f"Last bucket snapshot MEM: {last.snapshot_memory_time}, DISK: {last.snapshot_disk_time}"
+                )
+    except Exception as e:
+        handle_api_error(e)
+
+
 # ─────────────────────────────────────────────────────────────
 #  Image Commands
 # ─────────────────────────────────────────────────────────────
@@ -327,10 +737,15 @@ def snapshot():
     help="Filter snapshots by metadata (format: key=value)",
     multiple=True,
 )
+@click.option(
+    "--interactive", "-i", is_flag=True, help="Interactive pagination with arrow keys"
+)
+@click.option("--page", type=int, default=None, help="Page number (1-based)")
+@click.option("--limit", type=int, default=None, help="Page size")
 @click.option(
     "--json/--no-json", "json_mode", default=False, help="Output in JSON format"
 )
-def list_snapshots(metadata, json_mode):
+def list_snapshots(metadata, interactive, page, limit, json_mode):
     """List all snapshots."""
     client = get_client()
     try:
@@ -338,11 +753,80 @@ def list_snapshots(metadata, json_mode):
         for meta in metadata:
             key, value = meta.split("=", 1)
             metadata_dict[key] = value
-        snapshots = client.snapshots.list(metadata=metadata_dict)
-        if json_mode:
-            for snap in snapshots:
-                click.echo(format_json(snap))
+        if interactive:
+            # Maintain dynamic search filters in interactive mode (do not mutate base metadata)
+            base_metadata = dict(metadata_dict)
+            search_filter: dict[str, str] = {}
+
+            def get_data(**kwargs):
+                effective_metadata = {**base_metadata, **search_filter}
+                return client.snapshots.list_paginated(
+                    metadata=effective_metadata, **kwargs
+                )
+
+            def format_item(snap):
+                created = unix_timestamp_to_datetime(snap.created)
+                return (
+                    f"   📸 {snap.id:<24} | {created:<22} | "
+                    f"{snap.status:<8} | vCPU {snap.spec.vcpus:<2} | Mem {snap.spec.memory:<6} | Disk {snap.spec.disk_size:<6} | img {snap.refs.image_id}"
+                )
+
+            def on_search():
+                # Prompt user for metadata search term: key=value or empty to clear
+                new_search = click.prompt(
+                    "Enter metadata filter as key=value (Enter to clear)", default=""
+                ).strip()
+                nonlocal search_filter  # type: ignore
+                if not new_search:
+                    search_filter = {}
+                else:
+                    if "=" not in new_search:
+                        click.echo("Invalid format. Use key=value.")
+                    else:
+                        k, v = new_search.split("=", 1)
+                        search_filter = {k.strip(): v.strip()}
+                return True
+
+            def search_summary():
+                if not search_filter:
+                    return ""
+                k, v = next(iter(search_filter.items()))
+                return f"{k}={v}"
+
+            _interactive_pagination(
+                get_data,
+                format_item,
+                items_attr="snapshots",
+                title="MorphCloud - Snapshot List",
+                limit=(limit or 50),
+                initial_page=(page or 1),
+                enable_search=True,
+                on_search=on_search,
+                search_summary_fn=search_summary,
+            )
+            return
+
+        # Non-interactive: if page/limit provided, use paginated endpoint, else full list
+        if page or limit:
+            resp = client.snapshots.list_paginated(
+                metadata=metadata_dict, page=(page or 1), limit=(limit or 50)
+            )
+            snapshots = resp.snapshots
         else:
+            snapshots = client.snapshots.list(metadata=metadata_dict)
+        if json_mode:
+            if page or limit:
+                # For paginated requests, emit the full envelope
+                click.echo(format_json(resp))
+            else:
+                for snap in snapshots:
+                    click.echo(format_json(snap))
+        else:
+            # Show page info only when paginated flags are used
+            if page or limit:
+                click.echo(
+                    f"Total snapshots: {resp.total} | Page {resp.page} of {resp.total_pages} (showing {len(snapshots)})"
+                )
             headers = [
                 "ID",
                 "Created At",
@@ -532,10 +1016,15 @@ def instance():
 @click.option(
     "--metadata", "-m", help="Filter instances by metadata (key=value)", multiple=True
 )
+@click.option("--page", type=int, default=None, help="Page number (1-based)")
+@click.option("--limit", type=int, default=None, help="Page size")
+@click.option(
+    "--interactive", "-i", is_flag=True, help="Interactive pagination with arrow keys"
+)
 @click.option(
     "--json/--no-json", "json_mode", default=False, help="Output in JSON format"
 )
-def list_instances(metadata, json_mode):
+def list_instances(metadata, page, limit, interactive, json_mode):
     """List all instances."""
     client = get_client()
     try:
@@ -544,11 +1033,80 @@ def list_instances(metadata, json_mode):
             key, value = meta.split("=", 1)
             metadata_dict[key] = value
 
-        instances = client.instances.list(metadata=metadata_dict)
-        if json_mode:
-            for inst in instances:
-                click.echo(format_json(inst))
+        if interactive:
+            # Maintain dynamic search filter (does not mutate base --metadata)
+            base_metadata = dict(metadata_dict)
+            search_filter: dict[str, str] = {}
+
+            def get_data(**kwargs):
+                effective_metadata = {**base_metadata, **search_filter}
+                return client.instances.list_paginated(
+                    metadata=effective_metadata, **kwargs
+                )
+
+            def format_item(inst):
+                created = unix_timestamp_to_datetime(inst.created)
+                http_services = ",".join(
+                    f"{svc.name}:{svc.port}" for svc in inst.networking.http_services
+                )
+                return (
+                    f"   🖥  {inst.id:<24} | {created:<22} | {inst.status:<8} | "
+                    f"vCPU {inst.spec.vcpus:<2} | Mem {inst.spec.memory:<6} | Disk {inst.spec.disk_size:<6} | Snapshot {inst.refs.snapshot_id} | {http_services}"
+                )
+
+            def on_search():
+                new_search = click.prompt(
+                    "Enter metadata filter as key=value (Enter to clear)", default=""
+                ).strip()
+                nonlocal search_filter  # type: ignore
+                if not new_search:
+                    search_filter = {}
+                else:
+                    if "=" not in new_search:
+                        click.echo("Invalid format. Use key=value.")
+                    else:
+                        k, v = new_search.split("=", 1)
+                        search_filter = {k.strip(): v.strip()}
+                return True
+
+            def search_summary():
+                if not search_filter:
+                    return ""
+                k, v = next(iter(search_filter.items()))
+                return f"{k}={v}"
+
+            _interactive_pagination(
+                get_data,
+                format_item,
+                items_attr="instances",
+                title="MorphCloud - Instance List",
+                limit=(limit or 50),
+                initial_page=(page or 1),
+                enable_search=True,
+                on_search=on_search,
+                search_summary_fn=search_summary,
+            )
+            return
+
+        # Non-interactive: support optional page/limit
+        if page or limit:
+            resp = client.instances.list_paginated(
+                metadata=metadata_dict, page=(page or 1), limit=(limit or 50)
+            )
+            instances = resp.instances
         else:
+            instances = client.instances.list(metadata=metadata_dict)
+        if json_mode:
+            if page or limit:
+                click.echo(format_json(resp))
+            else:
+                for inst in instances:
+                    click.echo(format_json(inst))
+        else:
+            if page or limit:
+                click.echo(
+                    f"Total instances: {resp.total} | Page {resp.page} of {resp.total_pages} (showing {len(instances)})"
+                )
             headers = [
                 "ID",
                 "Snapshot ID",
@@ -1071,7 +1629,12 @@ def set_instance_metadata(instance_id, metadata, metadata_args):
 
 @instance.command("set-ttl")
 @click.argument("instance_id")
-@click.option("--ttl-seconds", type=int, required=True, help="Time-to-live in seconds.")
+@click.option(
+    "--ttl-seconds",
+    type=int,
+    required=True,
+    help="Time-to-live in seconds. Use -1 to remove TTL.",
+)
 @click.option(
     "--ttl-action",
     type=click.Choice(["stop", "pause"]),
@@ -1081,8 +1644,9 @@ def set_instance_metadata(instance_id, metadata, metadata_args):
 )
 def set_instance_ttl(instance_id, ttl_seconds, ttl_action):
     """
-    Set a time-to-live (TTL) for an instance.
+    Set or remove a time-to-live (TTL) for an instance.
     The instance will be stopped or paused when the TTL expires.
+    Pass --ttl-seconds -1 to remove TTL.
     """
     client = get_client()
     try:
@@ -1094,14 +1658,26 @@ def set_instance_ttl(instance_id, ttl_seconds, ttl_action):
             )
             sys.exit(1)
 
+        removing = ttl_seconds == -1
+        spinner_text = (
+            f"Removing TTL for {instance_id}..."
+            if removing
+            else f"Setting TTL for {instance_id} to {ttl_seconds} seconds..."
+        )
+        success_text = (
+            "TTL removed successfully!" if removing else "TTL set successfully!"
+        )
         with Spinner(
-            text=f"Setting TTL for {instance_id} to {ttl_seconds} seconds...",
-            success_text="TTL set successfully!",
+            text=spinner_text,
+            success_text=success_text,
             success_emoji="⏳",
         ):
-            instance_obj.set_ttl(ttl_seconds, ttl_action)
+            instance_obj.set_ttl(None if removing else ttl_seconds, ttl_action)
 
-        click.echo(f"TTL set for {instance_id}: {ttl_seconds} seconds")
+        if removing:
+            click.echo(f"TTL removed for {instance_id}")
+        else:
+            click.echo(f"TTL set for {instance_id}: {ttl_seconds} seconds")
     except api.ApiError as e:
         if e.status_code == 404:
             click.echo(f"Error: Instance '{instance_id}' not found.", err=True)
@@ -1472,41 +2048,6 @@ def chat(instance_id, conversation_file, instructions):
         handle_api_error(e)
 
 
-@instance.command("computer-mcp")
-@click.argument("instance_id")
-def computer(instance_id):
-    """Start an interactive MCP computer session with an instance."""
-    client = get_client()
-    try:
-        from morphcloud.computer import Computer
-
-        instance = client.instances.get(instance_id)
-
-        computer = Computer(instance)
-
-        computer.mcp().run()
-    except ImportError as e:
-        click.echo(
-            f"Error: Computer requires additional dependencies (e.g., 'mcp'). {e}",
-            err=True,
-        )
-        sys.exit(1)
-    except TimeoutError:
-        click.echo(
-            f"Error: Timed out waiting for instance '{instance_id}' to be ready.",
-            err=True,
-        )
-        sys.exit(1)
-    except api.ApiError as e:
-        if e.status_code == 404:
-            click.echo(f"Error: Instance '{instance_id}' not found.", err=True)
-            sys.exit(1)
-        else:
-            handle_api_error(e)
-    except Exception as e:
-        handle_api_error(e)
-
-
 @instance.command("boot")
 @click.argument("snapshot_id")
 @click.option("--vcpus", type=int, required=False)
